import json
import os
import urllib
from base64 import b64encode

def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)

def load_schema(entity):
    with open(get_abs_path('schemas/{}.json'.format(entity))) as file:
        return json.load(file)

def authenticate(api_key, session):
    encoded_key = b64encode(bytes("{0}".format(api_key), 'utf-8')).decode("ascii")
    session.headers.update({ 'Authorization': 'Basic {0}'.format(encoded_key)})

def convert_events_to_events_schema(events):
    formatted_events = []
    for event in events['data']['values']:
        formatted_event = {}
        formatted_event['event'] = event
        formatted_event['values'] = {}
        for date in events['data']['values'][event]:
            formatted_event['values'][date] = events['data']['values'][event][date]
        formatted_events.append(formatted_event)
        
    return formatted_events

def build_event_query_param(events_to_get):
    event_query = '['
    for event in events_to_get:
        event_query += '"{0}",'.format(event)
    event_query = event_query[:-1] #strip off last comma
    event_query += "]"
    encoded_event_query = urllib.parse.quote(event_query)
    event_query_param = "&event=" + encoded_event_query

    return event_query_param

def build_date_query_param(state):
    from_date = state['events']['from_date']
    to_date = state['events']['to_date']
    state['events']['from_date'] = to_date
    query_string = '?type=general&unit=day&from_date={0}&to_date={1}'.format(from_date, to_date)

    return query_string

def build_query_string(events_to_get, state):
    if 'events' in state and state['events'] is not None:
        query_string = build_date_query_param(state)
    else:
        query_string = '?type=general&unit=day&interval=7'

    event_query_param = build_event_query_param(events_to_get)
    
    return query_string + event_query_param


import argparse
import requests
import singer
import json

from tap_mixpanel import utils

session = requests.Session()
logger = singer.get_logger()

def http_get(url):
    resp = session.request(method='get', url=url)
    return resp

def get_all_events(events_to_get, state):
    query_string = utils.build_query_string(events_to_get, state)
    response = http_get('https://mixpanel.com/api/2.0/events{0}'.format(query_string))
    events = response.json()

    formatted_events = utils.convert_events_to_events_schema(events)
    singer.write_records('events', formatted_events)
    
    return state

def sync_events(events_to_get, state):
    event_schema = utils.load_schema('events')
    singer.write_schema('events', event_schema, 'legend_size')
    state = get_all_events(events_to_get, state)
    singer.write_state(state)

def do_sync(config, state):
    logger.info("Starting Mixpanel sync")
    api_key = config['api-secret']
    events_to_get = config['event-names']
    utils.authenticate(api_key, session)

    if state:
        logger.info('Replicating events from {0} to {1}'.format(state['events']['from_date'], state['events']['to_date']))
    else:
        logger.info('Replicating all events from the past 7 days')

    sync_events(events_to_get, state)

    logger.info("Done Mixpanel sync")

def main():

    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-c', '--config', help='Config file', required=True)
    parser.add_argument(
        '-s', '--state', help='State file')

    args = parser.parse_args()

    with open(args.config) as config_file:
        config = json.load(config_file)

    missing_keys = []
    for key in ['api-secret', 'event-names']:
        if key not in config:
            missing_keys += [key]

    if len(missing_keys) > 0:
        logger.fatal("Missing required configuration keys: {}".format(missing_keys))
        exit(1)

    events_to_get = config['event-names']

    if not events_to_get:
        logger.fatal("No events specified in config.json")
        exit(1)

    state = {}
    if args.state:
        with open(args.state, 'r') as data_file:
            state = json.load(data_file)

    do_sync(config, state)
    
if __name__ == '__main__':
    main()
